@@ -1,9 +1,9 @@
 
-gDist.rtop = function(object, ...) {
-  params = object$params
+gDist.rtop = function(object, params = list(), ...) {
+  params = getRtopParams(object$params, newPar = params, ...)
   if (params$debug.level > 1) debug.level = params$debug.level else debug.level = 0
-  if (!"dObs" %in% names(object)) object$dObs = rtopDisc(object$observations, ...)
-  if (!"dPred" %in% names(object)) object$dPred = rtopDisc(object$predictionLocations, ...)
+  if (!"dObs" %in% names(object)) object$dObs = rtopDisc(object$observations, params = params, ...)
+  if (!"dPred" %in% names(object)) object$dPred = rtopDisc(object$predictionLocations, params = params, ...)
   object$gDistObs = gDist(object$dObs, object$dObs, debug.level = debug.level, params = params, ...)
   object$gDistPredObs = gDist(object$dObs, object$dPred, debug.level = debug.level, params = params, ...)
   object$gDistPred = gDist(object$dPred, object$dPred, diag=TRUE, debug.level = debug.level, params = params,...)
@@ -11,28 +11,12 @@ gDist.rtop = function(object, ...) {
 }
 
 
-gDist.SpatialLinesDataFrame = function(object, object2=NULL, ...) {
-  if (is(object2,"SpatialLinesDataFrame")) object2 = as(object2,"SpatialLines")
-  gDist(as(object,"SpatialLines"),object2)
-}
-
-
-gDist.SpatialLines = function(object, object2=NULL, ...) {
-  dObs = rtopDisc(object, ...)
-  gDistObs = gDist(dObs, ...)
-  if (!is.null(object2)) {
-    dPred = rtopDisc(object2, ...)
-    gDistPredObs = gDist(dObs, dPred, ...)
-    gDistPred = gDist(dPred, dPred, diag=TRUE,...)
-    list(gDistObs = gDistObs, gDistPred = gDistPred, gDistPredObs = gDistPredObs)
-  } else list(gDistObs = gDistObs)
-}
 
 
 
 gDist.SpatialPolygonsDataFrame = function(object, object2=NULL, ...) {
   if (is(object2,"SpatialPolygonsDataFrame")) object2 = as(object2,"SpatialPolygons")
-  gDist(as(object,"SpatialPolygons"),object2)
+  gDist(as(object,"SpatialPolygons"),object2, ...)
 }
 
 
@@ -42,7 +26,7 @@ gDist.SpatialPolygons = function(object, object2=NULL, ...) {
   if (!is.null(object2)) {
     dPred = rtopDisc(object2, ...)
     gDistPredObs = gDist(dObs, dPred, ...)
-    gDistPred = gDist(dPred, dPred, diag=TRUE,...)
+    gDistPred = gDist(dPred, dPred, diag=TRUE, ...)
     list(gDistObs = gDistObs, gDistPred = gDistPred, gDistPredObs = gDistPredObs)
   } else list(gDistObs = gDistObs)
 }
@@ -50,6 +34,7 @@ gDist.SpatialPolygons = function(object, object2=NULL, ...) {
 
 gDist.list = function(object,object2=NULL,diag = FALSE, debug.level = 0, ...) {
   variogramModel=list(model = "Gho",params = 0)
+  if (debug.level == 1) print("Creating Ghos distances. This can take some time")
   if (inherits(object[[1]], "SpatialPoints")) {
     gDist = varMat(object, object2,diag = diag,variogramModel = variogramModel, 
              debug.level = debug.level, ...)
