@@ -1,6 +1,7 @@
 # For compiling the Fortran file:
 # R CMD SHLIB vred.f
 
+
 createRtopObject = function(observations, predictionLocations,
    formulaString, params=list(), ainfo, areas, overlapObs, overlapPredObs, 
    ...) {
@@ -21,16 +22,16 @@ createRtopObject = function(observations, predictionLocations,
     stop("ainfo and areas cannot be given as arguments together with observations and/or predictionLocations, please check documentation")
   } else {
     if (!inherits(observations,"SpatialPolygonsDataFrame") && "obs" %in% names(dots)) 
-      observations = SpatialPolygonsDataFrame(observations,data = dots$obs)
+      if (!inherits(observations, "STS")) observations = SpatialPolygonsDataFrame(observations,data = dots$obs)
     if (!missing(predictionLocations) && !inherits(predictionLocations,"SpatialPolygonsDataFrame") && "pred" %in% names(dots)) 
-      predictionLocations = SpatialPolygonsDataFrame(predictionLocations,data = dots$pred)
+      if (!inherits(predictionLocations, "STS"))predictionLocations = SpatialPolygonsDataFrame(predictionLocations,data = dots$pred)
   }
   if (missing(observations)) stop("Observations are missing")
 #  if (missing(predictionLocations)) stop("predictionLocations are missing")
   if (!"area" %in% names(observations) && inherits(observations,"SpatialPolygons")) {
      observations$area = sapply(slot(observations, "polygons"), function(i) slot(i, "area"))
-#  } else if (!"length" %in% names(observations) && inherits(observations,"SpatialLines")) {
-#     observations$length = SpatialLinesLengths(observations)
+  } else if (inherits(observations, "STS") && !"area" %in% names(observations@sp)) {
+    observations@sp$area = sapply(slot(observations@sp, "polygons"), function(i) slot(i, "area"))
   }
   object$observations = observations
   if (!missing(predictionLocations)) {
@@ -44,6 +45,8 @@ createRtopObject = function(observations, predictionLocations,
           data = data.frame(area = areas), match.ID = TRUE)  
 #    } else if (!"length" %in% names(observations) && inherits(predictionLocations,"SpatialLines")) {
 #       predictionLocations$length = SpatialLinesLengths(predictionLocations)
+    } else if (inherits(predictionLocations, "STS") && !"area" %in% names(predictionLocations@sp)) {
+      predictionLocations@sp$area = sapply(slot(predictionLocations@sp, "polygons"), function(i) slot(i, "area"))      
     } 
     object$predictionLocations = predictionLocations
   }
@@ -65,12 +68,11 @@ createRtopObject = function(observations, predictionLocations,
   if (object$params$nugget) {
     if (!missing(overlapObs) && !is.null(overlapObs)) {
       object$overlapObs = overlapObs
-    } else object$overlapObs = findOverlap(observations, params = object$params, debug.level = object$params$debug.level)
+    } else object$overlapObs = findOverlap(observations, debug.level = object$params$debug.level)
     if (!missing(overlapPredObs) && !is.null(overlapPredObs)) {
       object$overlapPredObs = overlapPredObs
     } else if (!missing(predictionLocations)) 
-        object$overlapPredObs = findOverlap(observations,predictionLocations, 
-              params = object$params,  debug.level = object$params$debug.level)
+        object$overlapPredObs = findOverlap(observations,predictionLocations, debug.level = object$params$debug.level)
   }
   class(object) = "rtop" 
   object
@@ -136,7 +138,7 @@ getRtopDefaultParams = function(parInit,
    model="Ex1",
    nugget = FALSE,
    unc = TRUE,
-   rresol = 25,  # Resolution real areas
+   rresol = 100,  # Resolution real areas
    hresol = 5,    # Resolution in x-direction rectangles
 #   logtrans = FALSE, # Logtransform data
    cloud = FALSE,  # work with cloud variogram
@@ -150,20 +152,32 @@ getRtopDefaultParams = function(parInit,
                  #       7 - gstat fitting (Nj/hj^2)
                  #       8 - opposite of weighted least squares difference according to Cressie (1985) - err2=n*(ymod/yobs-1)^2
                  #       9 - Neutreal WLS-method - err = min(err2,err3)
-   gDistEst = TRUE, # use ghosh distance
-   gDistPred = TRUE,
+   gDistEst = FALSE, # use ghosh distance
+   gDistPred = FALSE,
    maxdist = Inf,
    nmax = 10,
    hstype = "regular", # Sampling type for hypothetical areas
 #   rstype = ifelse(!missing(observations) && inherits(observations,"SpatialLines"),"regular","rtop"), 
                   # Sampling type for real areas
    rstype = "rtop",
+   nclus = 1,
+   cnAreas = 100,
+   clusType = NULL,
    wlim = 1.5,
    wlimMethod = "all",
    cv = FALSE,
    debug.level = 1,
-   partialOverlap = FALSE,
-   olim = 1e-4){
+   observations,
+   formulaString
+   ){
+
+
+
+#if (!missing(observations) & missing(cutoff)) {
+#  x = coordinates(observations)[, 1]
+#  y = coordinates(observations)[, 2]
+#  cutoff = (0.35 * sqrt((max(x) - min(x))^2 + (max(y) - min(y))^2)/100)
+#}
 list(model = model, nugget = nugget, unc = unc, 
      rresol = rresol, hresol=hresol, rstype = rstype, hstype = hstype, 
 #     logtrans = logtrans, 
@@ -171,8 +185,9 @@ list(model = model, nugget = nugget, unc = unc,
 #     cutoff = cutoff, 
      amul = amul, dmul = dmul,
      fit.method = fit.method, gDistEst = gDistEst, gDistPred = gDistPred, 
-     maxdist = maxdist, nmax = nmax, wlim = wlim, wlimMethod = wlimMethod, cv = cv,
-     partialOverlap = partialOverlap, olim = olim,  debug.level = debug.level)
+     maxdist = maxdist, nmax = nmax, nclus = nclus, cnAreas = cnAreas, clusType = clusType,
+     wlim = wlim, wlimMethod = wlimMethod, cv = cv,
+     debug.level = debug.level)
 }
 
 ###########################################
@@ -200,11 +215,18 @@ findParInitDefault = function(model) {
 
 #########################################
 findParInit = function(formulaString,observations,model) {
-  vario = variogram (formulaString, observations)
+  if (inherits(observations, "STS")) {
+    ntime = dim(observations)[2]
+    observations = observations[sample(1:ntime, 20),]
+    vario = rtopVariogram(observations, formulaString = formulaString)
+    aObs = sapply(slot(observations@sp, "polygons"), function(i) slot(i, "area"))
+  } else {
+    vario = variogram (formulaString, observations)
+    aObs = sapply(slot(observations, "polygons"), function(i) slot(i, "area"))
+  }
   parInit = data.frame(parl=c(1:5),paru=1,par0 = 1)
   parInit[1,1] = min(vario$gamma)/10
   parInit[1,2] = max(vario$gamma)*500 
-  aObs = sapply(slot(observations, "polygons"), function(i) slot(i, "area"))
   parInit[2,1] = sqrt(min(aObs))/4
   parInit[2,2] = max(vario$dist)*10
   minla = min(aObs)
@@ -215,11 +237,16 @@ findParInit = function(formulaString,observations,model) {
   parInit[4,2] = 1.5
   parInit[5,1] = 0.1
   parInit[5,2] = 1.7
+  if (model == "Ex1") {
+    parInit[4,2] = 1
+    parInit[5,2] = 1  
+  }
+  
 
   parInit[,3] = sqrt(parInit[,1]*parInit[,2])
   if (model %in% c("Exp","Sph", "Gau")) {
     parInit = parInit[1:3,]
-  } else if (model == "Sp1" | model == "Ga1"){
+  } else if (model == "Sp1"){
     parInit = parInit[1:4,]
   } else if (model == "Ex1"){
     parInit = parInit
