@@ -1,11 +1,13 @@
 
 # varMat.rtop hardly uses the functions further below, should be shortened
 
-varMat.rtop = function(object, varMatUpdate = FALSE, ...) {
-  params = getRtopParams(object$params,  ...)
+varMat.rtop = function(object, varMatUpdate = FALSE, params = list(), ...) {
+  params = getRtopParams(object$params,  newPar = params, ...)
   observations = object$observations
+  if (is(observations, "STSDF")) observations = observations@sp
   nObs = dim(observations@data)[1]
   predictionLocations = object$predictionLocations
+  if (is(predictionLocations, "STSDF")) predictionLocations = predictionLocations@sp
   variogramModel = object$variogramModel
   lgDistPred = params$gDistPred
   maxDist = params$maxDist
@@ -30,10 +32,20 @@ varMat.rtop = function(object, varMatUpdate = FALSE, ...) {
         gDistObs = object$gDistObs
       } else {
         dObs = object$dObs
-        object$gDistObs = gDistObs = gDist(dObs)
+        object$gDistObs = gDistObs = gDist(dObs, params = params)
       }
-      varMatObs = matrix(mapply(FUN = varioEx,gDistObs,MoreArgs=list(variogramModel)),
+      if (!is.null(params$nclus) && params$nclus > 1 && nObs > params$cnAreas && requireNamespace("parallel")) {
+        cl = rtopCluster(params$nclus, type = params$clusType)
+        varMatObs = matrix(unlist(parallel::parLapply(cl, 1:nObs, fun = function(x, gDistObs, variogramModel) 
+          mapply(gDistObs[, x],
+                 FUN = function(y) varioEx(y, variogramModel)), 
+                           gDistObs = gDistObs, variogramModel = variogramModel)), 
+                           nrow = nObs, ncol = nObs)
+      } else {
+        varMatObs = matrix(mapply(gDistObs, FUN = varioEx, 
+                          MoreArgs=list(variogramModel)),
                           nrow = nObs,ncol = nObs)
+      }
       vDiagObs = diag(varMatObs)
       for (ia in 1:(nObs-1)) {
         for (ib in (ia+1):nObs) {
@@ -50,7 +62,8 @@ varMat.rtop = function(object, varMatUpdate = FALSE, ...) {
     attr(object$varMatObs, "variogramModel") = variogramModel
     obsComp = TRUE
   }
-  if (!params$cv && (!"varMatPredObs" %in% names(object) | varMatUpdate)) {
+  if (!params$cv && "predictionLocations" %in% names(object) && 
+        (!"varMatPredObs" %in% names(object) | varMatUpdate)) {
 #    ftype = ifelse(inherits(predictionLocations,"SpatialPolygons"),"polygons","lines")
     varMatObs = object$varMatObs
     vDiagObs = diag(varMatObs)
@@ -65,17 +78,28 @@ varMat.rtop = function(object, varMatUpdate = FALSE, ...) {
     if (lgDistPred) {
       if ("gDistPred" %in% names(object)) {
         gDistPred = object$gDistPred
-      } else object$gDistPred = gDistPred = gDist(dPred, diag=TRUE)
+      } else object$gDistPred = gDistPred = gDist(dPred, diag=TRUE, params = params)
       if ("gDistPredObs" %in% names(object)) {
         gDistPredObs = object$gDistPredObs
-      } else object$gDistPredObs = gDistPredObs = gDist(dObs,dPred)
+      } else object$gDistPredObs = gDistPredObs = gDist(dObs,dPred, params = params)
 
       print("Creating prediction semivariance matrix. This can take some time.")
       object$varMatPred = varMatPred = 
                 matrix(mapply(FUN = varioEx,gDistPred,MoreArgs=list(variogramModel)),
                           nrow = nPred,ncol = 1)
-      varMatPredObs = matrix(mapply(FUN = varioEx,gDistPredObs,MoreArgs=list(variogramModel)),
+      
+      if (!is.null(params$nclus) && params$nclus > 1 && nObs > params$cnAreas && requireNamespace("parallel")) {
+        cl = rtopCluster(nclus = params$nclus, type = params$clusType)
+
+        varMatPredObs = matrix(unlist(parallel::parLapply(cl, 1:nPred, fun = function(x, gDistPredObs, variogramModel) 
+          mapply(gDistPredObs[, x],
+                 FUN = function(y) varioEx(y, variogramModel)), 
+          gDistPredObs = gDistPredObs, variogramModel = variogramModel)), 
+          nrow = nObs, ncol = nPred)      
+      } else {
+        varMatPredObs = matrix(mapply(FUN = varioEx,gDistPredObs,MoreArgs=list(variogramModel)),
                           nrow = nObs,ncol = nPred)
+      }
       if (is.null(dim(varMatPred)) || dim(varMatPred)[1] != dim(varMatPred)[2]) 
           vDiagPred = varMatPred else vDiagPred = diag(varMatPred)
       for (ia in 1:nObs) {
@@ -127,11 +151,19 @@ varMat.rtop = function(object, varMatUpdate = FALSE, ...) {
 }
     
 
-varMat.matrix = function(object, variogramModel,  ...) {
+varMat.matrix = function(object, variogramModel, diag = FALSE, sub1, sub2, ...) {
   ndim = dim(object)[1] 
   mdim = dim(object)[2]
-  matrix(mapply(FUN = varioEx,object,MoreArgs=list(variogramModel)),
+  varMatrix = matrix(mapply(FUN = varioEx,object,MoreArgs=list(variogramModel)),
                           nrow = ndim,ncol = mdim)
+  if (diag) sub1 = sub2 = diag(varMatrix)
+  if (!missing(sub1) & !missing(sub2)) {  
+    for (ia in 1:ndim) {
+      for (ib in 1:mdim) if (!(diag & ia == ib)) varMatrix[ia,ib] = varMatrix[ia,ib] - 0.5*(sub1[ia] + sub2[ib])
+    }
+  }     
+  varMatrix
+  
 }
 
 
@@ -152,27 +184,32 @@ varMat.SpatialPolygons = function(object,object2 = NULL,variogramModel,
 
 
 varMatDefault = function(object1,object2 = NULL,variogramModel,
-     overlapObs, overlapPredObs, ...) {
+     overlapObs, overlapPredObs,  ...) {
   params = getRtopParams(...)
-  d1 = rtopDisc(object1,params) 
-  if (!is.null(object2)) d2 = rtopDisc(object2,params) 
+  d1 = rtopDisc(object1, params) 
+  if (!is.null(object2)) d2 = rtopDisc(object2, params) 
   if (params$gDistPred) {
-    gDist1 = gDist(d1)
-    varMat1 = varMat(gDist1, variogramModel = variogramModel, ...)
+    gDist1 = gDist(d1, params = params)
+# calling varMat.matrix
+    varMatObs = varMat(gDist1, variogramModel = variogramModel, params = params, diag = TRUE, ...)
   } else {
-    varMat1 = varMat(d1, variogramModel = variogramModel, ...)
+    varMatObs = varMat(d1, variogramModel = variogramModel, params = params, ...)
   }  
-  if (is.null(object2)) return(varMat1)
-  varMatObs = varMat1
+
+  if (is.null(object2)) return(varMatObs)
   
   if (params$gDistPred & !is.null(object2)) {
-    gDistPred = gDist(d2, diag = TRUE)
-    varMatPred = varMat(gDistPred, diag = TRUE, ...)
-    gDistPredObs = gDist(d1, d2)
-    varMatPredObs = varMat(gDist1,gDistPred,sub1 = diag(varMatObs),sub2 = varMatPred, ...)
+    gDistPred = gDist(d2, diag = TRUE, params = params)
+# Calling varMat.matrix
+    varMatPred = varMat(gDistPred, params = params, variogramModel = variogramModel, ...)
+    gDistPredObs = gDist(d1, d2, params = params)
+    varMatPredObs = varMat(gDistPredObs,sub1 = diag(varMatObs),sub2 = varMatPred, params = params, 
+                           variogramModel = variogramModel, ...)
   } else {
-    varMatPred = varMat(d2,diag=TRUE)
-    varMatPredObs = varMat(d1,d2,sub1 = diag(varMatObs),sub2 = varMatPred)
+# Calling varMat.list
+    varMatPred = varMat(d2,diag=TRUE, params = params, variogramModel = variogramModel, )
+    varMatPredObs = varMat(d1,d2,sub1 = diag(varMatObs),sub2 = varMatPred, params = params, 
+                           variogramModel = variogramModel, ...)
   }
   if (params$nugget) {
     if (missing(overlapObs)) 
@@ -210,6 +247,7 @@ varMatDefault = function(object1,object2 = NULL,variogramModel,
 # coor1 and coor2 are coordinates of the areas, used for maximum distance
 varMat.list = function(object, object2=NULL, coor1, coor2, maxdist = Inf, 
               variogramModel, diag=FALSE, sub1, sub2, debug.level = 1, ...) {
+  params = getRtopParams(...)
   d1 = object
   d2 = object2
   if (is.null(d2)) {
@@ -226,36 +264,71 @@ varMat.list = function(object, object2=NULL, coor1, coor2, maxdist = Inf,
   mdim = length(d2)
   varMatrix = matrix(-999,nrow = ndim,ncol = mdim)
 
-  t0 = proc.time()[[3]]
-  for (ia in 1:ndim) {
-    t1 = proc.time()[[3]]
-    a1 = coordinates(d1[[ia]])
-    ip1 = dim(a1)[1]
-    first = ifelse(equal,ia,1)
-    lorder = c(first:mdim)
-    if (!missing(coor1) && ! missing(coor2) && maxdist < Inf) 
-         lorder = lorder[spDistsN1(coor2[first:mdim,],coor1[ia,]) < maxdist]
-    if (length(lorder) > 0) {
-      ld = d2[lorder]
-      lmat = mapply(vred,a2 = ld,MoreArgs = list(vredTyp="ind",a1 = a1,variogramModel = variogramModel))
-#      lmat = lvmat[[1]]
+  if (!is.null(params$nclus) && params$nclus > 1 && length(d1) + length(d2) > params$cnAreas && requireNamespace("parallel")) {
+#    cl = rtopCluster(params$nclus, {require(sp); vred = rtop:::vred}, type = params$clusType)
+    cl = rtopCluster(params$nclus, type = params$clusType)
+    if (missing(coor1)) coor1 = NULL
+    if (missing(coor2)) coor2 = NULL
+
+    
+    fun = function(ia, d1, d2, coor1, coor2, equal, maxdist) {    
+      ndim = length(d1)
+      mdim = length(d2)
+      a1 = coordinates(d1[[ia]])
+      ip1 = dim(a1)[1]
+      first = ifelse(equal,ia,1)
+      lorder = c(first:mdim)
+      if (!is.null(coor1) && ! is.null(coor2) && maxdist < Inf) 
+          lorder = lorder[spDistsN1(coor2[first:mdim,],coor1[ia,]) < maxdist]
+        if (length(lorder) > 0) {
+          ld = d2[lorder]
+          lmat = mapply(vred,a2 = ld,MoreArgs = list(vredTyp="ind",a1 = a1,variogramModel = variogramModel))
+        } else lmat = -999  
+      list(lmat, lorder)
+    }    
+    vmll = parallel::clusterApply(cl, 1:length(d1), d1 = d1, d2 = d2, coor1 = coor1, coor2 = coor2, 
+                       equal = equal, maxdist = maxdist, fun = fun)
+
+    for (ia in 1:length(vmll)) {
+      lmat = vmll[[ia]][[1]]
+      lorder = vmll[[ia]][[2]]
       if (!equal) varMatrix[ia,] = lmat else {
         varMatrix[ia,lorder] = lmat
         varMatrix[lorder,ia] = lmat
       }
     }
-#      print(acdf)
-    t2 = proc.time()[[3]]
-    if (debug.level > 0) print(paste("varMat - Finished element ",ia," out of ", ndim," in ", round(t2-t1,3),
-          "seconds - totally", round(t2-t0)," seconds"))
-  }
-  if (equal & variogramModel$model != "Gho") {
-    vDiag = diag(varMatrix)
-    for (ia in 1:(ndim-1)) {
-      for (ib in (ia+1):ndim) {
-        varMatrix[ia,ib] = varMatrix[ia,ib] - 0.5*(vDiag[ia] + vDiag[ib])
-        varMatrix[ib,ia] = varMatrix[ia,ib]
+  } else {
+    t0 = proc.time()[[3]]
+    for (ia in 1:ndim) {
+      t1 = proc.time()[[3]]
+      a1 = coordinates(d1[[ia]])
+      ip1 = dim(a1)[1]
+      first = ifelse(equal,ia,1)
+      lorder = c(first:mdim)
+      if (!missing(coor1) && ! missing(coor2) && maxdist < Inf) 
+           lorder = lorder[spDistsN1(coor2[first:mdim,],coor1[ia,]) < maxdist]
+      if (length(lorder) > 0) {
+        ld = d2[lorder]
+        lmat = mapply(vred,a2 = ld,MoreArgs = list(vredTyp="ind",a1 = a1,variogramModel = variogramModel))
+#        lmat = lvmat[[1]]
+        if (!equal) varMatrix[ia,] = lmat else {
+          varMatrix[ia,lorder] = lmat
+          varMatrix[lorder,ia] = lmat
+        }
       }
+#        print(acdf)
+      t2 = proc.time()[[3]]
+      if (debug.level > 0) print(paste("varMat - Finished element ",ia," out of ", ndim," in ", round(t2-t1,3),
+            "seconds - totally", round(t2-t0)," seconds"))
+    }
+  }
+    if (equal & variogramModel$model != "Gho") {
+      vDiag = diag(varMatrix)
+      for (ia in 1:(ndim-1)) {
+        for (ib in (ia+1):ndim) {
+          varMatrix[ia,ib] = varMatrix[ia,ib] - 0.5*(vDiag[ia] + vDiag[ib])
+          varMatrix[ib,ia] = varMatrix[ia,ib]
+        }
     }
   } else if (!missing(sub1) & !missing(sub2)) {  
     for (ia in 1:ndim) {
@@ -267,3 +340,11 @@ varMat.list = function(object, object2=NULL, coor1, coor2, maxdist = Inf,
 }
   
 
+
+varMat.STS = function(object, object2 = NULL, variogramModel, overlapObs, overlapPredObs, ...) {
+  if (is(object2,"STS")) object2 = object2@sp
+  object = object@sp
+  if (!is(object, "SpatialPolygons") || (!is.null(object2) && !inherits(object2, "SpatialPolygons")))
+    stop(paste("Cannot create covariance matrix from objects of class", class(object), class(object2))) 
+  varMat(object, object2, variogramModel, overlapObs, overlapPredObs, ...)
+}
\ No newline at end of file
