@@ -23,11 +23,21 @@ predictionLocations = readOGR(rpath, "predictionLocations")
   #rtopObj = checkVario(rtopObj)
   rtopObj$variogramModel                                                                        
   rtopObj2 = rtopKrige(rtopObj, cv = TRUE)
+
+
   print(attr(rtopObj2$varMatObs,"variogramModel"))
   
   rtopObj3 = rtopKrige(rtopObj)
-  rtopObj4 = rtopKrige(rtopObj2)
-  
+
+
+ varmat = varMat(observations, predictionLocations, variogramModel = rtopObj$variogramModel, 
+                 gDistEst = TRUE, gDistPred = TRUE, rresol = 25, hresol = 3)
+
+all.equal(varmat$varMatObs, rtopObj2$varMatObs)
+rtopObj4 = rtopKrige(rtopObj2)
+
+#debug(rtop:::rtopDisc.SpatialPolygons)
+  rtopObj5 = rtopKrige(rtopObj, params = list(cnAreas = 5, cDlim = 10, nclus = 2))
   
   print(summary(rtopObj2$predictions))
   print(summary(rtopObj3$predictions))
@@ -39,21 +49,36 @@ predictionLocations = readOGR(rpath, "predictionLocations")
   #spplot(rtopObj2$predictions,col.regions = bpy.colors(), c("observed","var1.pred"))
   print(cor(rtopObj2$predictions$observed,rtopObj2$predictions$var1.pred))
   
-  
+
+
 
   set.seed(1501)
+  library(intamap)
   useRtopWithIntamap()
   library(intamap)
   output = interpolate(observations,predictionLocations,
      optList = list(formulaString = obs~1, gDist = TRUE, cloud = FALSE, nmax = 10, rresol = 25, hresol = 3), 
         methodName = "rtop")
   
-  print(all.equal(rtopObj4$predictions@data$var1.pred, output$predictions@data$var1.pred))
+
+print(all.equal(rtopObj4$predictions@data$var1.pred, output$predictions@data$var1.pred))
   print(all.equal(rtopObj4$predictions@data$var1.var, output$predictions@data$var1.var))
 
+
 # Updating variogramModel
   
   rtopObj5 = varMat(rtopObj4)
   rtopObj6 = updateRtopVariogram(rtopObj5, exp = 1.5, action = "mult")
   rtopObj7 = varMat(rtopObj6)
 
+
+rtopObj10 = rtopSim(rtopObj, nsim = 5)
+rtopObj11 = rtopObj
+rtopObj11$predictionLocations = rtopObj11$observations
+rtopObj11$observations = NULL
+rtopObj12 = rtopSim(rtopObj11, nsim = 10, beta = 0.01)
+
+rtopObj10$simulations@data
+rtopObj12$simulations@data
+
+
